      MODULE s_CRDVEL
      CONTAINS

C*
      SUBROUTINE CRDVEL(NSTAT,XSTAT,XVEL,VFOUND,TIMNEQ,TIMFIX)
CC
CC NAME       :  CRDVEL
CC
CC PURPOSE    :  APPLY VELOCITY MODEL (XVEL) TO COORDINATES (XSTAT)
CC               TO TRANSFORM FROM COORDINATE EPOCH (TIMFIX) TO THE
CC               WISHED EPOCH (TIMNEQ)
CC
CC REMARKS    :  XSTAT AND XVEL ARE ALREADY ORDERED IN THE SAME KIND
CC
CC PARAMETERS :
CC         IN :  NSTAT  : NUMBER OF STATIONS
CC               XSTAT(K,I),K=1,2,3;I=1,..,NSTAT: RECTANGULAR R*8
CC                        STATION COORDINATES
CC               XVEL(K,I),K=1,3,I=1,NSTAT: ACCORDING
CC                        VELOCITIES                          R*8(*,*)
CC               TIMNEQ : TIME IN MJD OF INDIVIDUAL NORMAL    R*8
CC                        EQUATION
CC               TIMFIX : TIME OF REFERENCE COORDINATES       R*8
CC                        IN MJD
CC
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  E. BROCKMANN
CC
CC VERSION    :  3.5
CC
CC CREATED    :  16-JAN-94 11:00
CC
CC CHANGES    :  03-JAN-96 : EB: CHECK TIMFIX, TIMNEQ
CC               21-JUN-05 : MM: COMLFNUM.inc REMOVED, m_bern ADDED
CC               23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC               26-JAN-11 : RD: PROPAGATE ONLY IF XBVEL IS KNOWN
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1994     UNIVERSITY OF BERN
CC               SWITZERLAND
C*
      USE m_bern
      USE s_exitrc
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      INTEGER*4 ISTAT , NSTAT
C
      REAL*8    FACINT, TIMFIX, TIMNEQ
C
CCC       IMPLICIT  REAL*8  (A-H,O-Z)
C
C
C DECLARATIONS
C ------------
C
      REAL*8        XVEL(3,*),XSTAT(3,*)
      LOGICAL       VFOUND(*)
C COMMONS
C
C WARNING, IF TIMNEQ OR TIMFIX .EQ. 0
      IF (TIMFIX.EQ.0.D0.AND.TIMNEQ.EQ.0.D0) THEN
        WRITE(LFNERR,901)
901     FORMAT(/,' *** SR CRDVEL: WARNING: REFERENCE EPOCH AND',/,
     1                16X,'INPUT EPOCH IS ZERO',/,
     2                16X,'VELOCIITIES ARE NOT APPLIED',/)
      ENDIF
      IF (TIMFIX.EQ.0.D0.AND.TIMNEQ.NE.0.D0) THEN
        WRITE(LFNERR,902)
902     FORMAT(/,' *** SR CRDVEL: REFERENCE EPOCH IS ZERO',/)
        CALL EXITRC(2)
      ENDIF
      IF (TIMNEQ.EQ.0.D0.AND.TIMFIX.NE.0.D0) THEN
        WRITE(LFNERR,903)
903     FORMAT(/,' *** SR CRDVEL: INPUT EPOCH IS ZERO',/)
        CALL EXITRC(2)
      ENDIF
C
C INTERPOLATION FACTOR
      FACINT=(TIMNEQ-TIMFIX)/365.25D0
C
      DO 100 ISTAT=1,NSTAT
        IF (VFOUND(ISTAT)) THEN
          XSTAT(1,ISTAT)=XSTAT(1,ISTAT)+FACINT*XVEL(1,ISTAT)
          XSTAT(2,ISTAT)=XSTAT(2,ISTAT)+FACINT*XVEL(2,ISTAT)
          XSTAT(3,ISTAT)=XSTAT(3,ISTAT)+FACINT*XVEL(3,ISTAT)
        ENDIF
100   CONTINUE
C
      RETURN
C
      END SUBROUTINE

      END MODULE
