      MODULE s_GRAVSM
      CONTAINS

C*
      SUBROUTINE GRAVSM(GM,YS,YCB,F)
CC
CC NAME       :  GRAVSM
CC
CC PURPOSE    :  COMPUTE THE GRAVITATIONAL ATTRACTION EXCERTED BY A
CC               CELESTIAL BODY ON A EARTH SATELLITE AND ADD THIS
CC               FORCE TO THE RESULTING FORCE F(I),I=1,2,3
CC
CC PARAMETERS :
CC         IN :  GM     : GRAVIT. CONSTANT * MASS OF            R*8
CC                        DISTURBING BODY
CC               YS     : GEOCENTRIC POSITION OF THE SATELLITE  R*8
CC               YCB    : GEOCENTRIC POSITION (YCB(I),I=1,2,3)  R*8
CC                        AND DISTANCE (YCB(4)) OF THE DIS-
CC                        TURBING BODY
CC        OUT :  F(I),I=1,2,3: RESULTING FORCE ACTING ON        R*8
CC                        THE SATELLITE
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  87/12/10 18:57
CC
CC CHANGES    :  23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      INTEGER*4 I
C
      REAL*8    DELTA , DELTA3, GM    , R3
C
CCC       IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 YS(*),YCB(*),F(*)
      DELTA=0.D0
      DO 10 I=1,3
10    DELTA=DELTA+(YS(I)-YCB(I))**2
      DELTA3=DSQRT(DELTA)**3
      R3=YCB(4)**3
      DO 20 I=1,3
20    F(I)=F(I)-GM*((YS(I)-YCB(I))/DELTA3+YCB(I)/R3)
      RETURN
      END SUBROUTINE

      END MODULE
