      MODULE s_METEX1
      CONTAINS

C*
      SUBROUTINE METEX1(TS,PS,HS,HEIGHT,T,P,H)
CC
CC NAME       :  METEX1
CC
CC PURPOSE    :  EXTRAPOLATE METEO-DATA TO "HEIGHT" FROM SEA-LEVEL
CC               VALUES IMPROVED VERSION (WITH RESPECT TO METEXT)
CC               FROM A.GEIGER, ETH ZUERICH
CC
CC PARAMETERS :
CC         IN :  TS     : TEMPERATURE     ON SEA LEVEL        R*8
CC               PS     : PRESSURE (MBAR) ON SEA LEVEL        R*8
CC               HS     : HUMIDITY (%)    ON SEA LEVEL        R*8
CC               HEIGHT : STATION HEIGHT                      R*8
CC        OUT :  T      : TEMPERATURE     AT HEIGHT "HEIGHT"  R*8
CC               P      : PRESSURE (MBAR) AT HEIGHT "HEIGHT"  R*8
CC               H      : HUMIDITY (%)    AT HEIGHT "HEIGHT"  R*8
CC
CC REMARK     :  T, P ARE EXTRAPOLATED USING SOME BAROMETRIC HEIGHT-
CC               FORMULAS, H IS DEFINED TO GIVE THE SAME WATER-VAPOUR -
CC               PRESSURE AT HEIGHT "HEIGHT" AS ON SEA LEVEL.
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  87/11/02 15:43
CC
CC CHANGES    :  02-MAR-96 : JJ/MR: CHECK FOR EXTREMELY NEGATIVE
CC                                   HEIGHTS
CC               10-NOV-97 : SS : ... AND EXTREMLY POSITIVE HEIGHTS
CC               23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      REAL*8    H     , HEIGHT, HS    , P     , PS    , T     , TS
C
CCC       IMPLICIT REAL*8 (A-H,O-Z)
C
      IF (HEIGHT.GT.-10000.D0 .AND.
     1    HEIGHT.LT. 10000.D0) THEN
        P=PS*(1-2.26E-5*HEIGHT)**5.225
        T=TS-HEIGHT*.0065
        H=HS*EXP(-6.396E-4*HEIGHT)
      ELSE
        P=PS
        T=TS
        H=HS
      ENDIF
C
      RETURN
      END SUBROUTINE

      END MODULE
