      MODULE s_SUBDIV
      CONTAINS

C*
      SUBROUTINE SUBDIV(METH,N,X)
CC
CC NAME       :  SUBDIV
CC
CC PURPOSE    :  PARTITION OF INTERVAL (-1,+1) INTO N-1 INTERVALS
CC               METH=1: EQUIPARTITION
CC                   =2: ZEROES OF TSCHEBYSHEFF POLYNOMIALS
CC                   =3: ZEROES OF LEGENDRE POLYNOMIALS
CC
CC PARAMETERS :
CC         IN :  METH   : METHOD OF PARTITION (SEE ABOVE)       I*4
CC               N      : NUMBER OF POINTS                      I*4
CC        OUT :  X(I),I=1,2,..,N: RESULTING PARTITION           R*8
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  G.BEUTLER
CC
CC VERSION    :  3.4  (JAN 93)
CC
CC CREATED    :  88/11/03 10:37
CC
CC CHANGES    :  10-AUG-94 : MR: CALL EXITRC
CC               07-MAY-96 : MR: REMOVE UNUSED VARIABLES
CC               21-JUN-05 : MM: COMLFNUM.inc REMOVED, m_bern ADDED
CC               23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1987     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      USE m_bern
      USE s_exitrc
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      INTEGER*4 I     , IFIRST, METH  , N     , NTERM
C
      REAL*8    PI
C
CCC       IMPLICIT REAL*8 (A-H,O-Z)
CCC       IMPLICIT INTEGER*4 (I-N)
      REAL*8 X(*),LGROOT(12,6)
      REAL*8 LGR12(6),LGR16(8)
      DATA IFIRST/1/
C
C COMPUTE PI, SET ROOTS OF LEGENDRE POLYNOMIALS UPON FIRST CALL
      IF(IFIRST.EQ.1)THEN
        PI=4.D0*DATAN(1.D0)
        IFIRST=0
        LGROOT(1,1)=0.D0
C
        LGROOT(2,1)=.577350269189626D0
C
        LGROOT(3,1)=0.D0
        LGROOT(3,2)=.774596669241483D0
C
        LGROOT(4,1)=.339981043584856D0
        LGROOT(4,2)=.861136311594053D0
C
        LGROOT(5,1)=0.D0
        LGROOT(5,2)=.538469310105683D0
        LGROOT(5,3)=.906179845938664D0
C
        LGROOT(6,1)=.238619186083197D0
        LGROOT(6,2)=.661209386466265D0
        LGROOT(6,3)=.932469514203152D0
C
        LGROOT(7,1)=0.D0
        LGROOT(7,2)=.405845151377397D0
        LGROOT(7,3)=.741531185599394D0
        LGROOT(7,4)=.949107912342759D0
C
        LGROOT(8,1)=.183434642495650D0
        LGROOT(8,2)=.525532409916329D0
        LGROOT(8,3)=.796666477413627D0
        LGROOT(8,4)=.960289856497536D0
C
        LGROOT(9,1)=0.D0
        LGROOT(9,2)=.324253423403809D0
        LGROOT(9,3)=.613371432700590D0
        LGROOT(9,4)=.836031107322636D0
        LGROOT(9,5)=.968160239507626D0
C
        LGROOT(10,1)=.148874338981631D0
        LGROOT(10,2)=.433395394129247D0
        LGROOT(10,3)=.679409568299024D0
        LGROOT(10,4)=.865063366688985D0
        LGROOT(10,5)=.973906528517172D0
C
        LGR12(1)=.125233408511469D0
        LGR12(2)=.367831498998180D0
        LGR12(3)=.587317954286617D0
        LGR12(4)=.769902674194305D0
        LGR12(5)=.904117256370475D0
        LGR12(6)=.981560634246719D0
C
        LGR16(1)=.095012509837637440185D0
        LGR16(2)=.281603350779258913230D0
        LGR16(3)=.458016777657227386342D0
        LGR16(4)=.617876244402643748447D0
        LGR16(5)=.755404408355003033895D0
        LGR16(6)=.865631202387831743880D0
        LGR16(7)=.944575023073232576078D0
        LGR16(8)=.989400934991649932596D0
      END IF
C
C EQUIPARTITION
      IF(N.EQ.1)THEN
        X(1)=0.D0
      ELSE IF(METH.EQ.1)THEN
        DO 10 I=1,N
          X(I)=-1.D0+(I-1)*2.D0/(N-1)
10      CONTINUE
C
C TSCHEBYSHEFF
      ELSE IF(METH.EQ.2)THEN
        DO 30 I=1,N
          X(N+1-I)=DCOS((I-0.5D0)*PI/N)
30      CONTINUE
C
C LEGENDRE
      ELSE IF(METH.EQ.3)THEN
        IF(N.GT.10.AND.(N.NE.12.AND.N.NE.16))THEN
          WRITE(LFNERR,35)N
35        FORMAT(/,' *** SR SUBDIV: NUMBER OF POINTS FOR METH=3 ',
     1             'ILLEGAL:',I3,/,
     2             16X,'ONLY N<11, N=12, N=16 LEGAL',/)
          CALL EXITRC(2)
        ELSE
          NTERM=(N-1)/2+1
          IF((N/2)*2-N.EQ.0)THEN
            DO 40 I=1,NTERM
              IF(N.LE.10)THEN
                X(I)=LGROOT(N,I)
                X(NTERM+I)=-X(I)
              ELSE IF(N.EQ.12)THEN
                X(I)=LGR12(I)
                X(NTERM+I)=-X(I)
              ELSE IF(N.EQ.16)THEN
                X(I)=LGR16(I)
                X(NTERM+I)=-X(I)
              END IF
40          CONTINUE
          ELSE
            X(1)=LGROOT(N,1)
            DO 50 I=2,NTERM
              X(I)=LGROOT(N,I)
              X(NTERM+I-1)=-X(I)
50          CONTINUE
          END IF
        END IF
      ELSE
        WRITE(LFNERR,20)METH
20      FORMAT(/,' *** SR SUBDIV: ILLEGAL METHOD =',I3,/)
        CALL EXITRC(2)
      END IF
      RETURN
      END SUBROUTINE

      END MODULE
