      MODULE s_WGTELV
      CONTAINS

C*
      SUBROUTINE WGTELV(ICOELV,ELEVA,COVI)
CC
CC NAME       :  WGTELV
CC
CC PURPOSE    :  COMPUTE ELEVATION-DEPENDENT OBS. VARIANCES
CC               ON ZERO-DIFFERENCE LEVEL.
CC
CC PARAMETERS :
CC         IN :  ICOELV : MODEL FOR ELEV.-DEP. OBS. WEIGHTING I*4
CC                        =0: EQUAL WEIGHTING FOR ALL OBS.
CC                        =1: 1/SIN(ELEVA)**2
CC                        =2: 1/SIN(ELEVA)**4
CC               ELEVA  : SATELLITE ELEVATION (IN DEGREES)    R*8
CC        OUT :  COVI   : ELEVATION-DEPENDENT VARIANCE        R*8
CC                        NORMALIZED TO ZENITH (I.E. COVI=1
CC                        FOR ELEVA=90)
CC
CC REMARKS    :  ---
CC
CC AUTHOR     :  M.ROTHACHER, S.SCHAER
CC
CC VERSION    :  4.1
CC
CC CREATED    :  29-JAN-97
CC
CC CHANGES    :  28-JAN-03 : RS: COVI REAL*4 -> REAL*8
CC               16-JUN-05 : MM: COMCONST.inc REPLACED BY d_const
CC               21-JUN-05 : MM: COMLFNUM.inc REMOVED, m_bern ADDED
CC               23-JUN-05 : MM: IMPLICIT NONE AND DECLARATIONS ADDED
CC               18-JUN-07 : RD: NEW MODEL ICOELV=2: 1/SIN(ELEVA)**4
CC
CC COPYRIGHT  :  ASTRONOMICAL INSTITUTE
CC      1997     UNIVERSITY OF BERN
CC               SWITZERLAND
CC
C*
      USE m_bern
      USE d_const, ONLY: PI
      USE s_exitrc
      IMPLICIT NONE
C
C DECLARATIONS INSTEAD OF IMPLICIT
C --------------------------------
      INTEGER*4 ICOELV
C
      REAL*8    ELEVA
C
CCC       IMPLICIT REAL*8 (A-H,O-Z)
C
      REAL*8  COVI
C
C
C COMPUTE ELEVATION-DEPENDENT OBSERVATION VARIANCES
C -------------------------------------------------
      IF (ICOELV.EQ.0) THEN
C
C STANDARD MODEL: EQUAL WEIGHTING OF ALL OBSERVATIONS
        COVI=1.D0
C
C MODEL 1: 1/SIN(ELEVA)**2
      ELSEIF (ICOELV.EQ.1) THEN
        COVI=1.D0/DSIN(ELEVA/180.D0*PI)**2
C
C MODEL 2: 1/SIN(ELEVA)**4
      ELSEIF (ICOELV.EQ.2) THEN
        COVI=1.D0/DSIN(ELEVA/180.D0*PI)**4
      ELSE
        WRITE(LFNERR,901) ICOELV
901     FORMAT(/,' *** SR WGTELV: INVALID MODEL FOR ELEVATION-',
     1    'DEPENDENT OBSERVATION WEIGHTING',
     3    /,16X,'MODEL NUMBER SPECIFIED: ',I3,/)
        CALL EXITRC(2)
      END IF
C
      RETURN
      END SUBROUTINE

      END MODULE

